//
// LED_neopixel.h
//
// NeoPixel driving for the 12th Doctor's second Sonic Screwdriver
//
#ifndef _LED_NEOPIXEL_H_
#define _LED_NEOPIXEL_H_

//---------------------
// LED output routines
//
void ledSetup();
inline void sendByte( uint8_t val ) __attribute__((always_inline));
void sendLeds( struct LED_PIXEL *leds, struct LED_RGB *pixelBuf, uint8_t brightness, uint8_t nPixels );

// 12th Doctor's 2nd Sonic has 5 x RGB LEDs
//
#define PIXELS 5

struct LED_PIXEL{           // colour and brightness of each LED
  uint8_t colour;           // colour index into palette tables
  uint8_t bright;           // linear brightness (this is gamma corrected before output)
};

struct LED_RGB{             // raw pixel buffer required by sendLeds()
  uint8_t r;
  uint8_t g;
  uint8_t b;
};

//-----------------------------------------------------------------
// an 8-bit rainbow palette
// - fully saturated colours red..green..blue from index 0x00..0xEF
// - 0xF0= white, 0xF1..0xFE= pastel shades, 0xFF= black (off)
// - the source should contain a colours.html file showing a table of these colours
//
const PROGMEM uint8_t pal_red[] = {
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF8, 0xF2, 0xEB, 0xE5, 0xDF, 0xD8, 0xD2, 
  0xCB, 0xC5, 0xBF, 0xB8, 0xB2, 0xAC, 0xA5, 0x9F, 0x99, 0x92, 0x8C, 0x85, 0x7F, 0x79, 0x72, 0x6C, 
  0x65, 0x5F, 0x59, 0x52, 0x4C, 0x46, 0x3F, 0x39, 0x32, 0x2C, 0x26, 0x1F, 0x19, 0x13, 0x0C, 0x06, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x06, 0x0C, 0x13, 0x19, 0x1F, 0x26, 0x2C, 0x32, 0x39, 0x3F, 0x46, 0x4C, 0x52, 0x59, 0x5F, 
  0x66, 0x6C, 0x72, 0x79, 0x7F, 0x85, 0x8C, 0x92, 0x98, 0x9F, 0xA5, 0xAC, 0xB2, 0xB8, 0xBF, 0xC5, 
  0xCC, 0xD2, 0xD8, 0xDF, 0xE5, 0xEB, 0xF2, 0xF8, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFE, 0xFE, 0xFE, 0xE1, 0xB6, 0x99, 0x99, 0x99, 0x99, 0x99, 0xB6, 0xE1, 0xFE, 0xFE, 0x00
};

const PROGMEM uint8_t pal_green[] = {
  0x00, 0x06, 0x0C, 0x13, 0x19, 0x1F, 0x26, 0x2C, 0x33, 0x39, 0x3F, 0x46, 0x4C, 0x52, 0x59, 0x5F, 
  0x66, 0x6C, 0x72, 0x79, 0x7F, 0x85, 0x8C, 0x92, 0x99, 0x9F, 0xA5, 0xAC, 0xB2, 0xB8, 0xBF, 0xC5, 
  0xCC, 0xD2, 0xD8, 0xDF, 0xE5, 0xEB, 0xF2, 0xF8, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF8, 0xF2, 0xEB, 0xE5, 0xDF, 0xD8, 0xD2, 
  0xCB, 0xC5, 0xBF, 0xB8, 0xB2, 0xAC, 0xA5, 0x9F, 0x98, 0x92, 0x8C, 0x85, 0x7F, 0x79, 0x72, 0x6C, 
  0x65, 0x5F, 0x59, 0x52, 0x4C, 0x46, 0x3F, 0x39, 0x33, 0x2C, 0x26, 0x1F, 0x19, 0x13, 0x0C, 0x06, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0xFF, 0x99, 0xC4, 0xF0, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xD3, 0xA7, 0x99, 0x99, 0x99, 0x99, 0x00
};
const PROGMEM uint8_t pal_blue[] = {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x06, 0x0C, 0x13, 0x19, 0x1F, 0x26, 0x2C, 0x33, 0x39, 0x3F, 0x46, 0x4C, 0x52, 0x59, 0x5F, 
  0x66, 0x6C, 0x72, 0x79, 0x7F, 0x85, 0x8C, 0x92, 0x98, 0x9F, 0xA5, 0xAC, 0xB2, 0xB8, 0xBF, 0xC5, 
  0xCB, 0xD2, 0xD8, 0xDF, 0xE5, 0xEB, 0xF2, 0xF8, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF8, 0xF2, 0xEB, 0xE5, 0xDF, 0xD8, 0xD2, 
  0xCC, 0xC5, 0xBF, 0xB8, 0xB2, 0xAC, 0xA5, 0x9F, 0x99, 0x92, 0x8C, 0x85, 0x7F, 0x79, 0x72, 0x6C, 
  0x66, 0x5F, 0x59, 0x52, 0x4C, 0x46, 0x3F, 0x39, 0x33, 0x2C, 0x26, 0x1F, 0x19, 0x13, 0x0C, 0x06, 
  0xFF, 0x99, 0x99, 0x99, 0x99, 0x99, 0xA7, 0xD3, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xF0, 0xC4, 0x00
}; 

// some colours...
#define BLACK   255
#define WHITE   240
#define RED     0
#define ORANGE  20
#define YELLOW  40
#define GREEN   80
#define CYAN    120
#define BLUE    160
#define PURPLE  180
#define LILAC   200

// Brightness gamma correction table (x^2)
//
const PROGMEM uint8_t gammaTab[256]={
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,1,1,1,1,1,1,2,2,2,2,2,3,3,3,3,
 4,4,4,4,5,5,5,5,6,6,6,7,7,7,8,8,
 9,9,9,10,10,11,11,11,12,12,13,13,14,14,15,15,
 16,16,17,17,18,18,19,19,20,20,21,22,22,23,23,24,
 25,25,26,27,27,28,29,29,30,31,31,32,33,33,34,35,
 36,36,37,38,39,40,40,41,42,43,44,44,45,46,47,48,
 49,50,50,51,52,53,54,55,56,57,58,59,60,61,62,63,
 64,65,66,67,68,69,70,71,72,73,74,75,76,77,79,80,
 81,82,83,84,85,87,88,89,90,91,93,94,95,96,97,99,
 100,101,102,104,105,106,108,109,110,112,113,114,116,117,118,120,
 121,122,124,125,127,128,129,131,132,134,135,137,138,140,141,143,
 144,146,147,149,150,152,153,155,156,158,160,161,163,164,166,168,
 169,171,172,174,176,177,179,181,182,184,186,188,189,191,193,195,
 196,198,200,202,203,205,207,209,211,212,214,216,218,220,222,224,
 225,227,229,231,233,235,237,239,241,243,245,247,249,251,253,255
};

#endif /* _LED_NEOPIXEL_H_ */


