//
// LSM9DS0 accelerometer,gyro,magnetometer (on FLORA 9-DOF board)
//
// simple interface in C, lots taken from Adafruit_LSM9DS0
// TonyWilk
//
#include "accel.h"

#ifdef __cplusplus
extern "C"{
#endif

I2C_SLAVE LSM;



//--------------------------------------------
// initialise I2C and test comms to LSM device
//
I2C_STATUS accel_init()
{
  uint8_t c;
  I2C_init(I2C_FM);                           // I2C Fast Mode (400KHz - depending on pullups)
  I2C_slave_init( &LSM, 0x00, I2C_8B_REG );   // (0x00 is a dummy address)

  if( read8( ACCEL_ADDR, LSM9DS0_REGISTER_WHO_AM_I_XM, &c ) != I2C_OK )
	  return I2C_NACK;
  
  // Enable the accelerometer continous
  write8(ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG1_XM, 0x67); // 100hz XYZ
  write8(ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG5_XM, 0b11110000);
  // enable mag continuous
  write8(ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG7_XM, 0b00000000);
  // enable gyro continuous
  write8(GYRO_ADDR, LSM9DS0_REGISTER_CTRL_REG1_G, 0x0F); // on XYZ
  // enable the temperature sensor (output rate same as the mag sensor)
  read8(ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG5_XM, &c);
  write8(ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG5_XM, c | (1<<7));

  // Set default ranges for the various sensors  
  // Accelerometer
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG2_XM, &c );
  c= (c & 0b11000111) | LSM9DS0_ACCELRANGE_2G;
  write8( ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG2_XM, c );

  // Magnetometer
  read8(ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG6_XM, &c );
  c= (c & 0b10011111) | LSM9DS0_MAGGAIN_2GAUSS;
  write8(ACCEL_ADDR, LSM9DS0_REGISTER_CTRL_REG6_XM, c );

  // Gyro
  read8(GYRO_ADDR, LSM9DS0_REGISTER_CTRL_REG4_G, &c );
  c= (c & 0b11001111) | LSM9DS0_GYROSCALE_245DPS;
  return write8(GYRO_ADDR, LSM9DS0_REGISTER_CTRL_REG4_G, c );
}


//--------------------------------
// read single byte from register
//
I2C_STATUS read8( uint8_t slaveAddr, uint8_t regAddr, uint8_t *dest )
{
  uint8_t c;
  I2C_slave_set_addr( &LSM, slaveAddr );
  return I2C_read( &LSM, regAddr, dest, 1 );
}

//--------------------------------
// write single byte to register
//
I2C_STATUS write8( uint8_t slaveAddr, uint8_t regAddr, uint8_t data )
{
  I2C_slave_set_addr( &LSM, slaveAddr );
  return I2C_write( &LSM, regAddr, &data, 1 );
}


//--------------------------------
// Read acceleration
//
void readAccel( struct XYZ *xyz )
{
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_X_L_A, &xyz->X.hilo.lo );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_X_H_A, &xyz->X.hilo.hi );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_Y_L_A, &xyz->Y.hilo.lo );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_Y_H_A, &xyz->Y.hilo.hi );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_Z_L_A, &xyz->Z.hilo.lo );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_Z_H_A, &xyz->Z.hilo.hi );
}

//--------------------------------
// Read Gyro
//
void readGyro( struct XYZ *xyz)
{
  read8( GYRO_ADDR, LSM9DS0_REGISTER_OUT_X_L_G, &xyz->X.hilo.lo );
  read8( GYRO_ADDR, LSM9DS0_REGISTER_OUT_X_H_G, &xyz->X.hilo.hi );
  read8( GYRO_ADDR, LSM9DS0_REGISTER_OUT_Y_L_G, &xyz->Y.hilo.lo );
  read8( GYRO_ADDR, LSM9DS0_REGISTER_OUT_Y_H_G, &xyz->Y.hilo.hi );
  read8( GYRO_ADDR, LSM9DS0_REGISTER_OUT_Z_L_G, &xyz->Z.hilo.lo );
  read8( GYRO_ADDR, LSM9DS0_REGISTER_OUT_Z_H_G, &xyz->Z.hilo.hi );
}

//--------------------------------
// Read magnetometer
//
int magXoffset= 0;
int magYoffset= 4000;
int magZoffset= 0;

void readMag( struct XYZ *xyz )
{
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_X_L_M, &xyz->X.hilo.lo );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_X_H_M, &xyz->X.hilo.hi );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_Y_L_M, &xyz->Y.hilo.lo );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_Y_H_M, &xyz->Y.hilo.hi );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_Z_L_M, &xyz->Z.hilo.lo );
  read8( ACCEL_ADDR, LSM9DS0_REGISTER_OUT_Z_H_M, &xyz->Z.hilo.hi );

  // corrections...  (should calibrate this really)
  xyz->X.val = (int)(xyz->X.val) - magXoffset;
  xyz->Y.val = (int)(xyz->Y.val) - magYoffset;
  xyz->Z.val = (int)(xyz->Z.val) - magZoffset;
}

// Calculate heading in degrees from raw X,Y magnetometer values
// returns 0 .. +- 180
//
int calcHeading( struct XYZ *rawMag )
{
	float mgaussX, mgaussY;
	mgaussX= (int)(rawMag->X.val) * MAG_SCALE;
	mgaussY= (int)(rawMag->Y.val) * MAG_SCALE;
	
	return int(atan2( mgaussY, mgaussX ) * 180.0/M_PI );
}

uint8_t magCalMode= 0;		// 0=off, 1= initialise, 2=calibrating
int mcXmin,mcXmax,mcYmin,mcYmax,mcZmin,mcZmax;

void setMagCalMode( uint8_t mode )
{
	magCalMode= mode;
	switch( magCalMode )
	{
	case MAG_CAL_INIT:
		// init calibration - remove existing offsets
		magXoffset= magYoffset= magZoffset= 0;
		mcXmin= 30000;
		mcXmax= -30000;
		mcYmin= 30000;
		mcYmax= -30000;
		mcZmin= 30000;
		mcZmax= -30000;
		magCalMode= MAG_CAL_RUN;
		break;
	case MAG_CAL_STOP:
		// calc offset for midpoint on each axis
		magXoffset= (mcXmax + mcXmin) / 2;
		magYoffset= (mcYmax + mcYmin) / 2;
		magZoffset= (mcZmax + mcZmin) / 2;
		magCalMode= MAG_CAL_OFF;
		break;
	default:
		break;
	}
}

void processMagCal( struct XYZ *xyz )
{
	if( magCalMode ==  MAG_CAL_RUN )
	{
		// run calibration - get min/max for each axis
		if( (int)(xyz->X.val) > mcXmax ) mcXmax= (int)(xyz->X.val);
		if( (int)(xyz->X.val) < mcXmin ) mcXmin= (int)(xyz->X.val);
		if( (int)(xyz->Y.val) > mcYmax ) mcYmax= (int)(xyz->Y.val);
		if( (int)(xyz->Y.val) < mcYmin ) mcYmin= (int)(xyz->Y.val);
		if( (int)(xyz->Z.val) > mcZmax ) mcZmax= (int)(xyz->Z.val);
		if( (int)(xyz->Z.val) < mcZmin ) mcZmin= (int)(xyz->Z.val);
	}
}

void getMagCal( struct XYZ *xyz )
{
	xyz->X.val= magXoffset;
	xyz->Y.val= magYoffset;
	xyz->Z.val= magZoffset;
}




//----------------------------------------------------------
// processAccel
// - reads accelerometer
// - outputs:
//      axisRaw = raw acceleration along each axis (incl. gravity) scaled to mm/sec^2
//      axisAccel = accelerations along each axis (minus gravity) in mm/sec^2
//      axisDegrees = tilt of each axis in integer degrees (0=level +90 = up, -90 = down)
//
// Notes: 
//  - These are deliberately simple calculations, 
//    on a 16MHz ATmega828 (Arduino Pro Mini) this function takes a total of 1.8mS
//    860uS for the I2C at 100KHz, plus 940uS for the calculations
//  - acceleration values are in mm/sec^2, e.g. gravity = 9800 = 9.8m/s^2
//  - since we use this with no calibration the raw values < 2000 are noise, quick movments are usually > 3000
//  - the filtered axisAccel values are useful in detecting gestures
//  - any quick motion tends to affect all axes, jerking a hand-held device along one axis tends to
//    generate acceleration on that axis first, then the others, then the decelerations.
//    So implementing some hold-off time after detecting a 'jerky move' is a good idea  
//  - the axisDegrees angles are fairly stable unless the unit is jerked about
//    maybe could do with more filtering on the angle values ???

                                        // Exponential Moving Average filter...
#define FILTER_TIME_CONST   6.0         // time constant: time in samples to reach ~ 63% of a steady value
                                        // the rise time of a signal to > 90% is approx 2.5 times this value
                                        // at 15mS sampling a value of 4 == rise time of 150mS 
                                        
#define FILTER_WEIGHT (FILTER_TIME_CONST/(FILTER_TIME_CONST +1.0))

#define FILTER_TIME_CONST2  2.0
#define FILTER_WEIGHT2 (FILTER_TIME_CONST2/(FILTER_TIME_CONST2 +1.0))

struct XYZ accel;
struct XYZ gyro;


void processAccel( struct VECXYZ * axisRaw, struct VECXYZ * axisAccel, struct VECXYZ * axisDegrees )
{  
  static float x=0.0, y=0.0, z=0.0, lx=0.0, ly=0.0, lz=0.0;
  float r,t;

  readAccel( &accel );

  x= (FILTER_WEIGHT * x) + (1.0-FILTER_WEIGHT) * ((float)accel.X.val * ACCEL_SCALE);
  y= (FILTER_WEIGHT * y) + (1.0-FILTER_WEIGHT) * ((float)accel.Y.val * ACCEL_SCALE);
  z= (FILTER_WEIGHT * z) + (1.0-FILTER_WEIGHT) * ((float)accel.Z.val * ACCEL_SCALE);

  lx= (FILTER_WEIGHT2 * lx) + (1.0-FILTER_WEIGHT2) * x;   // 'long' values filtered twice
  ly= (FILTER_WEIGHT2 * ly) + (1.0-FILTER_WEIGHT2) * y;
  lz= (FILTER_WEIGHT2 * lz) + (1.0-FILTER_WEIGHT2) * z;

  
  axisRaw->x= int(x);
  axisRaw->y= int(y);
  axisRaw->z= int(z);
  
  r= sqrt( x*x + y*y + z*z);            // combined vector

  // trig: angle we want from combined vector to axis...  sin(angle) = x/r
  t= x/r;           
  axisDegrees->x = int(asin(t) * DEGREES_PER_RADIAN);  // tilt angle in degrees for axis

  // Relative acceleration for this axis
  // - subtract the 'longer term average' to remove gravity
  // - useful to detect jerks along an axis
  axisAccel->x= x - lx;

  // same again for y and z axes
  t= y/r;
  axisDegrees->y = int(asin(t) * DEGREES_PER_RADIAN);
  axisAccel->y= y - ly;
  t= z/r;
  axisDegrees->z = int(asin(t) * DEGREES_PER_RADIAN);
  axisAccel->z= z - lz;
}

#ifdef __cplusplus
}
#endif
/* end of accel.c */



