//
// themetune.h
//
// Doctor Who Theme Tune lookup
//
// Music taken from: 
//  Doctor Who Theme - 1963-1966     Ron Grainer
//  Transcription of the original Doctor Who theme music
//  by NotTon
//  from: https://musescore.com/nottonmusic/scores/1328796
//
// Had to adjust a couple of bits 'cos I'm only generating monophonic.
// - TonyWilk, 2017
//
// Notes:
// - for ease of use the table contains: NOTE (in Hz) and DURATION in 1/24th whole note (semibreve) units
// - this *must* be stored in RAM because we pre-process the values before playing
// - preprocessTune() converts the NOTE into phase increments and DURATION into no_of_sample_periods at
//   the required sampling frequency. This simplifies the sample output routine.
//
#define TEMPO 64          // TEMPO * D_noteDuration == millisecs

#define GAP   1           // inter-note gap in mS (currently don't use this)
#define NOGAP 0

int melody[]={    // Key of G  F#
/*
  NOTE_REST, D_TQUAVER,
  // Bar 1
  NOTE_REST, D_SBREVE,
  // Bar 2
  NOTE_REST, D_SBREVE,      You'll just have to imagine the
  // Bar 3                  derum-de-dum, derum-de-dum, dedi-di-di dum, dedi-di-di dum, etc.  
  NOTE_REST, D_SBREVE,      :)
  // Bar 4
  NOTE_REST, D_SBREVE,
  // Bar 5
  NOTE_REST, D_SBREVE,
  // Bar 6
  NOTE_REST, D_SBREVE,
  // Bar 7
  NOTE_REST, D_SBREVE,
*/  
  // Bar 8
  NOTE_REST, D_MINIM,
  NOTE_B4, D_CROTCHET, 
  NOTE_C7, D_CROTCHET,
  // Bar 9
  NOTE_B6 | NOTE_DECAY_LOW, D_SBREVE,
  // Bar 10
  NOTE_HOLD | NOTE_DECAY_LOW, D_MINIM,
  NOTE_D7, D_CROTCHET,
  NOTE_A5, D_CROTCHET,
  // Bar 11  
  NOTE_B5 | NOTE_DECAY_LOW, D_SBREVE,             // (should be B6 and B4 together)
  // Bar 12
  NOTE_HOLD | NOTE_DECAY_LOW, D_SBREVE,
  // Bar 13
  NOTE_B6, D_CROTCHET,
  NOTE_G6, D_CROTCHET,
  NOTE_B5, D_CROTCHET,
  NOTE_HOLD, D_TCROTCHET,
  NOTE_B5, D_TQUAVER,
  // Bar 14
  NOTE_D5, D_CROTCHET,
  NOTE_HOLD, D_TCROTCHET,
  NOTE_C6, D_TQUAVER,
  NOTE_B5 | NOTE_DECAY_LOW, D_MINIM,
  // Bar 15
  NOTE_HOLD | NOTE_DECAY_LOW, D_MINIM,
  NOTE_A4, D_CROTCHET,
  NOTE_B4, D_CROTCHET,
  NOTE_HOLD, D_TQUAVER,
  NOTE_C5, D_TQUAVER,
  NOTE_D5, D_TQUAVER,
  // Bar 16
  NOTE_B4 | NOTE_DECAY_LOW, D_MINIM,         
// NOTE_REST,D_TCROTCHET,
//  NOTE_C6, D_TQUAVER,
//  NOTE_B5, D_MINIM,
  // Bar 17
  NOTE_HOLD | NOTE_DECAY_LOW, D_SBREVE,
  // Bar 18
  NOTE_REST, D_SBREVE,
  // Bar 19
  NOTE_REST, D_MINIM,
  NOTE_A5, D_CROTCHET,
  NOTE_C7, D_CROTCHET,
  // Bar 20
  NOTE_B6 | NOTE_DECAY_LOW, D_SBREVE,
  // Bar 21
  NOTE_HOLD | NOTE_DECAY_LOW, D_MINIM,
  NOTE_G6, D_CROTCHET,
  NOTE_B6, D_CROTCHET,
  // Bar 22
  NOTE_A6, D_CROTCHET,
  NOTE_HOLD, D_TQUAVER,
  NOTE_G6, D_TQUAVER,
  NOTE_FS6, D_TQUAVER,
  NOTE_G6 | NOTE_DECAY_LOW, D_MINIM,
  // Bar 23
  NOTE_HOLD | NOTE_DECAY_LOW, D_MINIM,
  NOTE_G5, D_CROTCHET,
  NOTE_D7, D_CROTCHET,
  // Bar 24
  NOTE_E7, D_CROTCHET,
  NOTE_REST, D_TQUAVER,
  NOTE_D7, D_TQUAVER,
  NOTE_C7, D_TQUAVER,
  NOTE_D7, D_CROTCHET,
  NOTE_G6, D_TCROTCHET,
  NOTE_D7, D_TQUAVER,
  // Bar 25
  NOTE_E7, D_CROTCHET,
  NOTE_REST, D_TQUAVER,
  NOTE_D7, D_TQUAVER,
  NOTE_C7, D_TQUAVER,
  NOTE_D7, D_CROTCHET,
  NOTE_G6, D_TCROTCHET,
  NOTE_B6, D_TQUAVER,
  // Bar 26
  NOTE_A6 | NOTE_DECAY_LOW, D_MINIMDOT,
  NOTE_REST, D_TQUAVER,
  NOTE_G6, D_TQUAVER,
  NOTE_FS6, D_TQUAVER,
  // Bar 27
  NOTE_G6 | NOTE_DECAY_LOW, D_SBREVE,
  // Bar 28
  NOTE_HOLD | NOTE_DECAY_LOW, D_SBREVE,
  // Bar 29
  NOTE_HOLD | NOTE_DECAY_LOW, D_SBREVE,
  // Bar 30
  NOTE_HOLD | NOTE_DECAY_LOW, D_SBREVE,
  //
  NOTE_REST, D_SBREVE * 4,
  NOTE_END                  // End of table marker !
};

/*

// This was an attempt at the bassline
// - it used too much memory and didn't sound very good anyway
// - the tiny speaker doesn't do low notes very well
// - squarewave worked better than sine and adding note decay helped
//   but I never got anywhere near the old 'derum-de-dum' sound
// - gave up and stuck to just the melody.
//
// Left it commented out in case it's of any use in the future
// TonyWilk 
//
int bassline[]={
 NOTE_B2, D_TQUAVER,
  // Bar1
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  // Bar2
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_D3, D_TQUAVER,
  NOTE_E3, D_TCROTCHET,
  NOTE_E3, D_TQUAVER,
  NOTE_E3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  // Bar3
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  // Bar4
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_D3, D_TQUAVER,
  NOTE_E3, D_TCROTCHET,
  NOTE_E3, D_TQUAVER,
  NOTE_E3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  // Bar 5
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  // Bar 6
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_D3, D_TQUAVER,
  NOTE_E3, D_TCROTCHET,
  NOTE_E3, D_TCROTCHET,
  NOTE_D3, D_TQUAVER,
 NOTE_B2, D_TQUAVER,
  // Bar 7
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  // Bar 8
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_D3, D_TQUAVER,
  NOTE_E3, D_TCROTCHET,
  NOTE_E3, D_TCROTCHET,
  NOTE_D3, D_TQUAVER,
  NOTE_B2, D_TQUAVER,
  // Bar 9
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  // Bar 10
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TCROTCHET,
  NOTE_A2, D_TQUAVER,
 NOTE_B2, D_TQUAVER,
  // Bar 11
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  // Bar 12
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TCROTCHET,
  NOTE_A2, D_TQUAVER,
 NOTE_B2, D_TQUAVER,
  // Bar 13
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_D3, D_TQUAVER,
  // Bar 14
  NOTE_E3, D_TCROTCHET,
  NOTE_E3, D_TQUAVER,
  NOTE_E3, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TCROTCHET,
  NOTE_A2, D_TQUAVER,
 NOTE_C3, D_TQUAVER,
  // Bar 15
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_B2, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
  NOTE_C3, D_TQUAVER,
  NOTE_C3, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  // Bar 16
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  // Bar 17
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
 NOTE_A2, D_TQUAVER,
  NOTE_B2, D_TCROTCHET,
  NOTE_B2, D_TCROTCHET,
  NOTE_A2, D_TQUAVER,
 NOTE_FS2, D_TQUAVER,
  // Bar 18
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
 NOTE_FS2, D_TQUAVER,
  // Bar 19
  NOTE_G2, D_TCROTCHET,
  NOTE_G2, D_TQUAVER,
  NOTE_G2, D_TCROTCHET,
 NOTE_A2, D_TQUAVER,
  NOTE_B2, D_TCROTCHET,
  NOTE_B2, D_TCROTCHET,
  NOTE_A2, D_TQUAVER,
 NOTE_FS2, D_TQUAVER,
  // Bar 20
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TCROTCHET,
 NOTE_D2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TCROTCHET,
 NOTE_D2, D_TQUAVER,
  // Bar 21
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TCROTCHET,
 NOTE_D2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TCROTCHET,
 NOTE_A2, D_TQUAVER,
  // Bar 22
  NOTE_B2, D_TCROTCHET,
  NOTE_B2, D_TQUAVER,
  NOTE_B2, D_TCROTCHET,
 NOTE_D2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TCROTCHET,
 NOTE_D2, D_TQUAVER,
  // Bar 23
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TCROTCHET,
 NOTE_D2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TQUAVER,
  NOTE_E2, D_TCROTCHET,
 NOTE_G2, D_TQUAVER,
  //
  NOTE_REST, D_SBREVE * 4,
  NOTE_END
};
*/

